package gregapi.cover;

import static gregapi.data.CS.PX_N;
import static gregapi.data.CS.PX_P;
import gregapi.render.ITexture;

import java.util.List;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidTank;


/**
 * @author Gregorius Techneticies
 * 
 * Interfaces to use Cover Functionality.
 */
public interface ICover {
	/**
	 * Called when the Cover got successfully placed.
	 * @param aPlayer CAN be null!
	 */
	public void onCoverPlaced(byte aCoverSide, CoverData aData, Entity aPlayer, ItemStack aCover);
	
	/**
	 * Called when the Cover got successfully placed.
	 * @param aPlayer CAN be null!
	 */
	public void onCoverRemove(byte aCoverSide, CoverData aData, Entity aPlayer);
	
	/**
	 * @param aData CAN be null!
	 * @param aPlayer CAN be null!
	 * @return true to prevent that the cover gets placed.
	 */
	public boolean interceptCoverPlacement(byte aCoverSide, CoverData aData, Entity aPlayer);
	
	/**
	 * @param aPlayer CAN be null!
	 * @return true to prevent that the cover gets removed.
	 */
	public boolean interceptCoverRemoval(byte aCoverSide, CoverData aData, Entity aPlayer);
	
	/**
	 * Called before the Block does its own Tick. So for example before it checks a Recipe or something.
	 */
	public void onTickPre(byte aCoverSide, CoverData aData, long aTimer, boolean aIsServerSide, boolean aReceivedBlockUpdate, boolean aReceivedInventoryUpdate);
	
	/**
	 * Called after the Block does its own Tick. So for example after it has processed a Recipe or something.
	 */
	public void onTickPost(byte aCoverSide, CoverData aData, long aTimer, boolean aIsServerSide, boolean aReceivedBlockUpdate, boolean aReceivedInventoryUpdate);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block and cause the click animation
	 */
	public boolean onCoverClickedLeft(byte aCoverSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block without the click animation.
	 */
	public boolean interceptClickLeft(byte aCoverSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block and cause the click animation
	 */
	public boolean onCoverClickedRight(byte aCoverSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block without the click animation.
	 */
	public boolean interceptClickRight(byte aCoverSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Relaying the Tool Clicks to Covers.
	 */
	public long onToolClick(byte aCoverSide, CoverData aData, String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Tooltip List for this Cover Type.
	 */
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H);
	
	/**
	 * Called when some Entity walks over this Cover.
	 */
	public boolean onWalkOver(byte aCoverSide, CoverData aData, EntityLivingBase aEntity);
	
	/**
	 * Gets the Item that drops when removing the Cover.
	 */
	public ItemStack getCoverItem(byte aCoverSide, CoverData aData);
	
	/**
	 * @return if this side is Solid
	 */
	public boolean isSolid(byte aCoverSide, CoverData aData);
	
	/**
	 * @return if this side is Opaque
	 */
	public boolean isOpaque(byte aCoverSide, CoverData aData);
	
	/**
	 * @return if you need to save the visual state of this Cover to NBT because the visuals contain crucial information.
	 */
	public boolean needsVisualsSaved(byte aCoverSide, CoverData aData);
	
	public byte getRedstoneIn(byte aCoverSide, CoverData aData);
	public byte getRedstoneOutWeak(byte aCoverSide, CoverData aData, byte aDefaultRedstone);
	public byte getRedstoneOutStrong(byte aCoverSide, CoverData aData, byte aDefaultRedstone);
	
	public ITexture getCoverTextureSurface(byte aCoverSide, CoverData aData);
	public ITexture getCoverTextureAttachment(byte aCoverSide, CoverData aData, byte aTextureSide);
	public ITexture getCoverTextureHolder(byte aCoverSide, CoverData aData, byte aTextureSide);
	
	public Object getGUIServer(byte aCoverSide, CoverData aData, EntityPlayer aPlayer);
	public Object getGUIClient(byte aCoverSide, CoverData aData, EntityPlayer aPlayer);
	
	public float[] getCoverBounds(byte aCoverSide, CoverData aData);
	public float[] getHolderBounds(byte aCoverSide, CoverData aData);
	
	public static final float[][] BOXES_COVERS  = new float[][] {{PX_P[ 0], PX_P[ 0], PX_P[ 0], PX_N[ 0], PX_N[14], PX_N[ 0]}, {PX_P[ 0], PX_P[14], PX_P[ 0], PX_N[ 0], PX_N[ 0], PX_N[ 0]}, {PX_P[ 0], PX_P[ 0], PX_P[ 0], PX_N[ 0], PX_N[ 0], PX_N[14]}, {PX_P[ 0], PX_P[ 0], PX_P[14], PX_N[ 0], PX_N[ 0], PX_N[ 0]}, {PX_P[ 0], PX_P[ 0], PX_P[ 0], PX_N[14], PX_N[ 0], PX_N[ 0]}, {PX_P[14], PX_P[ 0], PX_P[ 0], PX_N[ 0], PX_N[ 0], PX_N[ 0]}};
	public static final float[][] BOXES_HOLDERS = new float[][] {{PX_P[ 7], PX_P[ 0], PX_P[ 7], PX_N[ 7], PX_N[ 8], PX_N[ 7]}, {PX_P[ 7], PX_P[ 8], PX_P[ 7], PX_N[ 7], PX_N[ 0], PX_N[ 7]}, {PX_P[ 7], PX_P[ 7], PX_P[ 0], PX_N[ 7], PX_N[ 7], PX_N[ 8]}, {PX_P[ 7], PX_P[ 7], PX_P[ 8], PX_N[ 7], PX_N[ 7], PX_N[ 0]}, {PX_P[ 0], PX_P[ 7], PX_P[ 7], PX_N[ 8], PX_N[ 7], PX_N[ 7]}, {PX_P[ 8], PX_P[ 7], PX_P[ 7], PX_N[ 0], PX_N[ 7], PX_N[ 7]}};
	
	public boolean interceptItemInsert(byte aCoverSide, CoverData aData, int aSlot, ItemStack aStack, byte aSide);
	public boolean interceptItemExtract(byte aCoverSide, CoverData aData, int aSlot, ItemStack aStack, byte aSide);
	public boolean getAccessibleSlotsFromSideOverride(byte aCoverSide, CoverData aData, byte aSide);
	public boolean canInsertItemOverride(byte aCoverSide, CoverData aData, int aSlot, ItemStack aStack, byte aSide);
	public boolean canExtractItemOverride(byte aCoverSide, CoverData aData, int aSlot, ItemStack aStack, byte aSide);
	public int[] getAccessibleSlotsFromSide(byte aCoverSide, CoverData aData, byte aSide, int[] aDefault);
	public boolean canInsertItem(byte aCoverSide, CoverData aData, int aSlot, ItemStack aStack, byte aSide);
	public boolean canExtractItem(byte aCoverSide, CoverData aData, int aSlot, ItemStack aStack, byte aSide);
	
	public boolean interceptFluidFill(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToFill);
	public boolean interceptFluidDrain(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToDrain);
	public boolean getFluidTankFillableOverride(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToFill);
	public boolean getFluidTankDrainableOverride(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToDrain);
	public boolean getFluidTanksOverride(byte aCoverSide, CoverData aData, byte aSide);
	public IFluidTank getFluidTankFillable(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToFill, IFluidTank aDefault);
	public IFluidTank getFluidTankDrainable(byte aCoverSide, CoverData aData, byte aSide, FluidStack aFluidToDrain, IFluidTank aDefault);
	public IFluidTank[] getFluidTanks(byte aCoverSide, CoverData aData, byte aSide, IFluidTank[] aDefault);
}