package gregapi.item.bumble;

import static gregapi.data.CS.BIOMES_DESERT;
import static gregapi.data.CS.BIOMES_MESA;
import static gregapi.data.CS.DEFAULT_ENVIRONMENT_TEMPERATURE;
import static gregapi.data.CS.T;
import gregapi.util.UT;

import java.util.Random;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;

/**
 * @author Gregorius Techneticies
 */
public interface IItemBumbleBee {
	/** @return The Amount of different Products for this Bumblebee Species. */
	public int bumbleProductCount(ItemStack aBumbleBee, short aMetaData);
	/** @return The Item produced by this Bumblebee Species. */
	public ItemStack bumbleProductStack(ItemStack aBumbleBee, short aMetaData, long aStacksize, int aProductIndex);
	/** @return 0 for no Chance, 10000 for a 100% Chance. */
	public int bumbleProductChance(ItemStack aBumbleBee, short aMetaData, int aProductIndex);
	/** @return true if the Environment allows the production of this specific Product. This is usually for checking for specific Flowers for this Product. */
	public boolean bumbleCanProduct(World aWorld, int aX, int aY, int aZ, ItemStack aBumbleBee, short aMetaData, int aProductIndex);
	
	/** @return true if the Environment allows the Bumblebee Species to be active at this point in time. This is used for the Main checks. */
	public boolean bumbleCanProduce(World aWorld, int aX, int aY, int aZ, ItemStack aBumbleBee, short aMetaData);
	
	/** @return 0 = Drone, 1 = Princess, 2 = Queen, 4 = Dead, 5 = Scanned Drone, 6 = Scanned Princess, 7 = Scanned Queen, 9 = Dead Scanned */
	public byte bumbleType(ItemStack aBumbleBee);
	/** @return ItemStack with the scanned variant of this Bumblebee. */
	public ItemStack bumbleScan(ItemStack aBumbleBee);
	/** @return ItemStack with the dead variant of this Bumblebee. */
	public ItemStack bumbleKill(ItemStack aBumbleBee);
	
	/** @return 0 for no Chance, 10000 for a 100% Chance. */
	public int bumbleMutateChance(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a mutated variant of this Bumblebee. Used for random Mutations. */
	public ItemStack bumbleMutate(ItemStack aBumbleBee, short aMetaData, Random aRandom);
	/** @return ItemStack with a special Offspring for this combination. This is only called in quarter of all cases for aBumbleBeeA (this), another quarter goes to aBumbleBeeB, the remaining half goes to selecting the same species as one of the Parents. */
	public ItemStack bumbleCombine(ItemStack aBumbleBeeA, short aMetaDataA, ItemStack aBumbleBeeB, short aMetaDataB, byte aBumbleType, Random aRandom);
	
	
	/** @return ItemStack with a Drone of this Species (and without NBT!) */
	public ItemStack bumbleDrone(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a Princess of this Species (and without NBT!) */
	public ItemStack bumblePrincess(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a Queen of this Species (and without NBT!) */
	public ItemStack bumbleQueen(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a dead Bumblebee of this Species (and without NBT!) */
	public ItemStack bumbleDead(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a scanned Drone of this Species (and without NBT!) */
	public ItemStack bumbleDrone_(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a scanned Princess of this Species (and without NBT!) */
	public ItemStack bumblePrincess_(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a scanned Queen of this Species (and without NBT!) */
	public ItemStack bumbleQueen_(ItemStack aBumbleBee, short aMetaData);
	/** @return ItemStack with a scanned dead Bumblebee of this Species (and without NBT!) */
	public ItemStack bumbleDead_(ItemStack aBumbleBee, short aMetaData);
	
	
	public static class Util {
		public static NBTTagCompound getBumbleTag(ItemStack aBumbleBee) {
			NBTTagCompound aNBT = UT.NBT.getOrCreate(aBumbleBee), rBumbleTag = aNBT.getCompoundTag("gt.bumble");
			aNBT.setTag("gt.bumble", rBumbleTag);
			return rBumbleTag;
		}
		
		public static ItemStack setBumbleTag(ItemStack aBumbleBee, NBTTagCompound aBumbleTag) {
			UT.NBT.getOrCreate(aBumbleBee).setTag("gt.bumble", aBumbleTag);
			return aBumbleBee;
		}
		
		public static NBTTagCompound getBumbleGenes(ItemStack aBumblePrincess, ItemStack aBumbleDrone, Random aRandom) {
			NBTTagCompound rBumbleTag = UT.NBT.make(), tBumbleTagA = getBumbleTag(aBumblePrincess), tBumbleTagB = getBumbleTag(aBumbleDrone);
			if (tBumbleTagA == null) {
				if (tBumbleTagB == null) {
					tBumbleTagA = getBumbleGenes(aRandom);
					tBumbleTagB = getBumbleGenes(aRandom);
				} else {
					tBumbleTagA = tBumbleTagB;
				}
			} else if (tBumbleTagB == null) {
				tBumbleTagB = tBumbleTagA;
			}
			setHumidityMin		(rBumbleTag, getHumidityMin		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setHumidityMax		(rBumbleTag, getHumidityMax		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setOffspring		(rBumbleTag, getOffspring		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setWorkForce		(rBumbleTag, getWorkForce		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setLifeSpan			(rBumbleTag, getLifeSpan		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setTemperatureMin	(rBumbleTag, getTemperatureMin	(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setTemperatureMax	(rBumbleTag, getTemperatureMax	(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setRainproof		(rBumbleTag, getRainproof		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setStormproof		(rBumbleTag, getStormproof		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setDayActive		(rBumbleTag, getDayActive		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setNightActive		(rBumbleTag, getNightActive		(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setOutsideActive	(rBumbleTag, getOutsideActive	(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			setInsideActive		(rBumbleTag, getInsideActive	(aRandom.nextBoolean()?tBumbleTagA:tBumbleTagB));
			return rBumbleTag;
		}
		
		public static NBTTagCompound getBumbleGenes(long aTemperature, BiomeGenBase aBiome, boolean aHasSky, Random aRandom) {
			NBTTagCompound rBumbleTag = UT.NBT.make();
			setHumidityMin(rBumbleTag, aBiome.rainfall - 0.05F - aRandom.nextInt(21)/100.0F);
			setHumidityMax(rBumbleTag, aBiome.rainfall + 0.05F + aRandom.nextInt(21)/100.0F);
			setTemperatureMin(rBumbleTag, aTemperature - 5 - aRandom.nextInt(11));
			setTemperatureMax(rBumbleTag, aTemperature + 5 + aRandom.nextInt(11));
			setOffspring(rBumbleTag,    1+aRandom.nextInt(     4));
			setWorkForce(rBumbleTag,    1+aRandom.nextInt( 10000));
			setLifeSpan	(rBumbleTag, 1200+aRandom.nextInt(142801));
			if (aHasSky) {
				setOutsideActive(rBumbleTag, T);
				if (aRandom.nextInt(10000) < aBiome.rainfall * 10000) setRainproof(rBumbleTag, T);
				if (aRandom.nextInt(20000) < aBiome.rainfall * 10000) setStormproof(rBumbleTag, T);
				if (BIOMES_DESERT.contains(aBiome.biomeName) || BIOMES_MESA.contains(aBiome.biomeName)) setNightActive(rBumbleTag, T); else setDayActive(rBumbleTag, T);
			} else {
				setInsideActive(rBumbleTag, T);
				if (aRandom.nextBoolean()) setNightActive(rBumbleTag, T); else setDayActive(rBumbleTag, T);
			}
			return rBumbleTag;
		}
		
		public static NBTTagCompound getBumbleGenes(Random aRandom) {
			NBTTagCompound rBumbleTag = UT.NBT.make();
			setHumidityMin(rBumbleTag, 0.45F - aRandom.nextInt(21)/100.0F);
			setHumidityMax(rBumbleTag, 0.55F + aRandom.nextInt(21)/100.0F);
			setTemperatureMin(rBumbleTag, DEFAULT_ENVIRONMENT_TEMPERATURE - 5 - aRandom.nextInt(11));
			setTemperatureMax(rBumbleTag, DEFAULT_ENVIRONMENT_TEMPERATURE + 5 + aRandom.nextInt(11));
			setOffspring(rBumbleTag,    1+aRandom.nextInt(     4));
			setWorkForce(rBumbleTag,    1+aRandom.nextInt( 10000));
			setLifeSpan	(rBumbleTag, 1200+aRandom.nextInt(142801));
			setOutsideActive(rBumbleTag, T);
			setDayActive(rBumbleTag, T);
			return rBumbleTag;
		}
		
		public static void setHumidityMin		(ItemStack aBumbleBee, float aHumidity)			{setHumidityMin		(getBumbleTag(aBumbleBee), aHumidity);}
		public static void setHumidityMax		(ItemStack aBumbleBee, float aHumidity)			{setHumidityMax		(getBumbleTag(aBumbleBee), aHumidity);}
		public static void setTemperatureMin	(ItemStack aBumbleBee, long aTemperature)		{setTemperatureMin	(getBumbleTag(aBumbleBee), aTemperature);}
		public static void setTemperatureMax	(ItemStack aBumbleBee, long aTemperature)		{setTemperatureMax	(getBumbleTag(aBumbleBee), aTemperature);}
		public static void setOffspring			(ItemStack aBumbleBee, long aOffspring)			{setOffspring		(getBumbleTag(aBumbleBee), aOffspring);}
		public static void setWorkForce			(ItemStack aBumbleBee, long aWorkForce)			{setWorkForce		(getBumbleTag(aBumbleBee), aWorkForce);}
		public static void setLifeSpan			(ItemStack aBumbleBee, long aLifeSpan)			{setLifeSpan		(getBumbleTag(aBumbleBee), aLifeSpan);}
		public static void setRainproof			(ItemStack aBumbleBee, boolean aRainproof)		{setRainproof		(getBumbleTag(aBumbleBee), aRainproof);}
		public static void setStormproof		(ItemStack aBumbleBee, boolean aStormproof)		{setStormproof		(getBumbleTag(aBumbleBee), aStormproof);}
		public static void setDayActive			(ItemStack aBumbleBee, boolean aDayActive)		{setDayActive		(getBumbleTag(aBumbleBee), aDayActive);}
		public static void setNightActive		(ItemStack aBumbleBee, boolean aNightActive)	{setNightActive		(getBumbleTag(aBumbleBee), aNightActive);}
		public static void setOutsideActive		(ItemStack aBumbleBee, boolean aOutsideActive)	{setOutsideActive	(getBumbleTag(aBumbleBee), aOutsideActive);}
		public static void setInsideActive		(ItemStack aBumbleBee, boolean aInsideActive)	{setInsideActive	(getBumbleTag(aBumbleBee), aInsideActive);}
		
		public static float getHumidityMin		(ItemStack aBumbleBee)	{return getHumidityMin(getBumbleTag(aBumbleBee));}
		public static float getHumidityMax		(ItemStack aBumbleBee)	{return getHumidityMax(getBumbleTag(aBumbleBee));}
		public static long getTemperatureMin	(ItemStack aBumbleBee)	{return getTemperatureMin(getBumbleTag(aBumbleBee));}
		public static long getTemperatureMax	(ItemStack aBumbleBee)	{return getTemperatureMax(getBumbleTag(aBumbleBee));}
		public static long getOffspring			(ItemStack aBumbleBee)	{return getOffspring(getBumbleTag(aBumbleBee));}
		public static long getWorkForce			(ItemStack aBumbleBee)	{return getWorkForce(getBumbleTag(aBumbleBee));}
		public static long getLifeSpan			(ItemStack aBumbleBee)	{return getLifeSpan(getBumbleTag(aBumbleBee));}
		public static boolean getRainproof		(ItemStack aBumbleBee)	{return getRainproof(getBumbleTag(aBumbleBee));}
		public static boolean getStormproof		(ItemStack aBumbleBee)	{return getStormproof(getBumbleTag(aBumbleBee));}
		public static boolean getDayActive		(ItemStack aBumbleBee)	{return getDayActive(getBumbleTag(aBumbleBee));}
		public static boolean getNightActive	(ItemStack aBumbleBee)	{return getNightActive(getBumbleTag(aBumbleBee));}
		public static boolean getOutsideActive	(ItemStack aBumbleBee)	{return getOutsideActive(getBumbleTag(aBumbleBee));}
		public static boolean getInsideActive	(ItemStack aBumbleBee)	{return getInsideActive(getBumbleTag(aBumbleBee));}
		
		public static void setHumidityMin		(NBTTagCompound aBumbleTag, float aHumidity)		{aBumbleTag.setFloat("minhum", aHumidity);}
		public static void setHumidityMax		(NBTTagCompound aBumbleTag, float aHumidity)		{aBumbleTag.setFloat("maxhum", aHumidity);}
		public static void setTemperatureMin	(NBTTagCompound aBumbleTag, long aTemperature)		{UT.NBT.setNumber( aBumbleTag, "mintemp"	, aTemperature);}
		public static void setTemperatureMax	(NBTTagCompound aBumbleTag, long aTemperature)		{UT.NBT.setNumber( aBumbleTag, "maxtemp"	, aTemperature);}
		public static void setOffspring			(NBTTagCompound aBumbleTag, long aOffspring)		{UT.NBT.setNumber( aBumbleTag, "offspring"	, UT.Code.bindStack(aOffspring));}
		public static void setWorkForce			(NBTTagCompound aBumbleTag, long aWorkForce)		{UT.NBT.setNumber( aBumbleTag, "work"		, UT.Code.bind(1, 10000, aWorkForce));}
		public static void setLifeSpan			(NBTTagCompound aBumbleTag, long aLifeSpan)			{UT.NBT.setNumber( aBumbleTag, "life"		, UT.Code.bind(1200, 144000, aLifeSpan));}
		public static void setRainproof			(NBTTagCompound aBumbleTag, boolean aRainproof)		{UT.NBT.setBoolean(aBumbleTag, "rain"		, aRainproof);}
		public static void setStormproof		(NBTTagCompound aBumbleTag, boolean aStormproof)	{UT.NBT.setBoolean(aBumbleTag, "storm"		, aStormproof);}
		public static void setDayActive			(NBTTagCompound aBumbleTag, boolean aDayActive)		{UT.NBT.setBoolean(aBumbleTag, "day"		, aDayActive);}
		public static void setNightActive		(NBTTagCompound aBumbleTag, boolean aNightActive)	{UT.NBT.setBoolean(aBumbleTag, "night"		, aNightActive);}
		public static void setOutsideActive		(NBTTagCompound aBumbleTag, boolean aOutsideActive)	{UT.NBT.setBoolean(aBumbleTag, "outside"	, aOutsideActive);}
		public static void setInsideActive		(NBTTagCompound aBumbleTag, boolean aInsideActive)	{UT.NBT.setBoolean(aBumbleTag, "inside"		, aInsideActive);}
		
		public static float getHumidityMin		(NBTTagCompound aBumbleTag)	{return aBumbleTag.getFloat("minhum");}
		public static float getHumidityMax		(NBTTagCompound aBumbleTag)	{return aBumbleTag.getFloat("maxhum");}
		public static long getTemperatureMin	(NBTTagCompound aBumbleTag)	{return aBumbleTag.getLong("mintemp");}
		public static long getTemperatureMax	(NBTTagCompound aBumbleTag)	{return aBumbleTag.getLong("maxtemp");}
		public static long getOffspring			(NBTTagCompound aBumbleTag)	{return UT.Code.bindStack(aBumbleTag.getLong("offspring"));}
		public static long getWorkForce			(NBTTagCompound aBumbleTag)	{return UT.Code.bind(1, 10000, aBumbleTag.getLong("work"));}
		public static long getLifeSpan			(NBTTagCompound aBumbleTag)	{return UT.Code.bind(1200, 144000, aBumbleTag.getLong("life"));}
		public static boolean getRainproof		(NBTTagCompound aBumbleTag)	{return aBumbleTag.getBoolean("rain");}
		public static boolean getStormproof		(NBTTagCompound aBumbleTag)	{return aBumbleTag.getBoolean("storm");}
		public static boolean getDayActive		(NBTTagCompound aBumbleTag)	{return aBumbleTag.getBoolean("day");}
		public static boolean getNightActive	(NBTTagCompound aBumbleTag)	{return aBumbleTag.getBoolean("night");}
		public static boolean getOutsideActive	(NBTTagCompound aBumbleTag)	{return aBumbleTag.getBoolean("outside");}
		public static boolean getInsideActive	(NBTTagCompound aBumbleTag)	{return aBumbleTag.getBoolean("inside");}
	}
}