package gregapi.worldgen;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.MultiTileEntityRegistry;
import gregapi.data.CS.BlocksGT;
import gregapi.data.CS.ConfigsGT;
import gregapi.data.OP;
import gregapi.oredict.OreDictManager;
import gregapi.oredict.OreDictMaterial;
import gregapi.util.ST;
import gregapi.util.UT;
import gregapi.util.WD;

import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

/**
 * @author Gregorius Techneticies
 */
public class Worldgen_GT_Ore_Bedrock extends WorldgenObject {
	public final int mProbability;
	public final OreDictMaterial mMaterial;
	public final boolean mOverworld, mNether, mEnd;
	
	public Worldgen_GT_Ore_Bedrock(String aName, boolean aDefault, int aProbability, boolean aOverworld, boolean aNether, boolean aEnd, OreDictMaterial aPrimary) {
		super(aName, Worldgenerator.sWorldgenList, aDefault);
		mOverworld			= 						ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Overworld"		, aOverworld);
		mNether				= 						ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Nether"			, aNether);
		mEnd				= 						ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "TheEnd"			, aEnd);
		mProbability		= Math.max(1,			ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Probability"		, aProbability));
		mMaterial			= OreDictMaterial.get(	ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Ore"				, aPrimary.mNameInternal));
		
		if (mEnabled) OreDictManager.INSTANCE.triggerVisibility("ore"+mMaterial.mNameInternal);
	}
	
	@Override
	public boolean executeWorldgen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		if (mMaterial.mID <= 0 || mMaterial.mID == W || !Worldgenerator.sCanGenerateUniqueFeatureForThisChunk || !isGenerationAllowed(aWorld, aDimensionType, (aDimensionType == -1 && mNether) || (aDimensionType == 0 && mOverworld) || (aDimensionType == +1 && mEnd) ? aDimensionType : ~aDimensionType) || aRandom.nextInt(mProbability) != 0) return F;
		if (aWorld.getBlock(aChunkX+8, 0, aChunkZ+8) == Blocks.bedrock) {
			Worldgenerator.sCanGenerateUniqueFeatureForThisChunk = F;
			
			
			ItemStack tRock = OP.rockGt.mat(mMaterial, 1);
			if (ST.valid(tRock)) {
				MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry("gt.multitileentity");
				if (tRegistry != null) {
					for (int i = 0; i < 16; i++) {
						int tX = aChunkX+aRandom.nextInt(32)-8, tZ = aChunkZ+aRandom.nextInt(32)-8;
						for (int tY = 90; tY > 62; tY--) {
							Block tContact = aWorld.getBlock(tX, tY, tZ);
							if (tContact.getMaterial().isLiquid()) break;
							if (!tContact.isOpaqueCube()) continue;
							if (tContact.getMaterial() != Material.grass && tContact.getMaterial() != Material.ground && tContact.getMaterial() != Material.sand && tContact.getMaterial() != Material.rock) break;
							if (WD.easyRep(aWorld, tX, tY+1, tZ)) tRegistry.mBlock.placeBlock(aWorld, tX, tY+1, tZ, SIDE_UNKNOWN, (short)32757, aRandom.nextInt(3)!=0?ST.save(UT.NBT.make(), NBT_VALUE, tRock):UT.NBT.make(), F, T);
							break;
						}
					}
				}
			}
			
			
			int[] tDistances = new int[] {0, 3, 6, 8, 9, 7, 4};
			for (int tY = 1; tY < 7; tY++) for (int tX = -tDistances[tY]; tX <= tDistances[tY]; tX++) for (int tZ = -tDistances[tY]; tZ <= tDistances[tY]; tZ++) {
				Worldgenerator.removeBedrock(aWorld, aChunkX+8+tX, tY, aChunkZ+8+tZ);
				switch(aRandom.nextInt(6)) {
				case 0: 		Worldgenerator.setOreBlock(aWorld, aChunkX+8+tX, tY, aChunkZ+8+tZ, mMaterial); break;
				case 1: case 2:	Worldgenerator.setSmallOreBlock(aWorld, aChunkX+8+tX, tY, aChunkZ+8+tZ, mMaterial); break;
				}
			}
			for (int tX = -2; tX <= 2; tX++) for (int tZ = -2; tZ <= 2; tZ++) {
				switch(aRandom.nextInt(6)) {
				case 0: 		BlocksGT.oreBedrock			.placeBlock(aWorld, aChunkX+8+tX, 0, aChunkZ+8+tZ, (byte)6, mMaterial.mID, null, F, T); break;
				case 1: case 2:	BlocksGT.oreSmallBedrock	.placeBlock(aWorld, aChunkX+8+tX, 0, aChunkZ+8+tZ, (byte)6, mMaterial.mID, null, F, T); break;
				}
			}
			// At least one Ore Block must be there. So place a large one in the Center.
			BlocksGT.oreBedrock.placeBlock(aWorld, aChunkX+8, 0, aChunkZ+8, (byte)6, mMaterial.mID, null, F, T);
		}
		return T;
	}
}